package com.sugamobile.realwear_voice_command

import android.app.Activity
import android.content.*
import android.view.View
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry

class RealwearVoiceCommandPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    companion object {
        private const val INTENT_ACTION_SPEECH_EVENT = "com.realwear.wearhf.intent.action.SPEECH_EVENT"

        private const val RECEIVING_EVENT_SET_COMMANDS = "set_commands"

        private const val SENDING_EVENT_COMMAND_RECEIVED = "command_received_event"

        private var activity: Activity? = null

        private lateinit var channel: MethodChannel
    }

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "realwear_voice_command")
        channel.setMethodCallHandler(this)
        flutterPluginBinding.getApplicationContext().registerReceiver(asrBroadcastReceiver, IntentFilter(INTENT_ACTION_SPEECH_EVENT))
    }

    override fun onAttachedToActivity(p0: ActivityPluginBinding) {
        activity = p0.activity
    }

    override fun onDetachedFromActivityForConfigChanges() {
        TODO("Not yet implemented")
    }

    override fun onReattachedToActivityForConfigChanges(p0: ActivityPluginBinding) {
        activity = p0.activity
    }

    override fun onDetachedFromActivity() {
        activity = null;
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
        when (call.method) {
            RECEIVING_EVENT_SET_COMMANDS -> {
                val commands = call.argument<String>("commands")
                setCommands(commands!!)
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    private fun setCommands(commands: String) {
        activity?.findViewById<View>(android.R.id.content)?.contentDescription = "hf_add_commands:${commands}"
    }

    private val asrBroadcastReceiver: BroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {
            val action = intent.action
            if (action == INTENT_ACTION_SPEECH_EVENT) {
                val command = intent.getStringExtra("command")
                channel.invokeMethod(SENDING_EVENT_COMMAND_RECEIVED, hashMapOf("command" to command))
            }
        }
    }
}
